/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.snmpgui;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import uk.co.petertribble.jingle.JingleTextPane;
import snmp.*;
import java.net.*;
import java.io.IOException;
import org.jfree.chart.*;

public class SnmpTreePanel extends JPanel implements TreeSelectionListener,
    ListSelectionListener, ActionListener
{

    private SnmpParams params;
    private JingleTextPane tp;
    private JingleTextPane tpdesc;
    private JPanel rpanel;
    private JPanel jp2;
    private SnmpChart chart;

    private JList slist;
    private DefaultListModel model;
    private JTree stree;
    private SnmpObject currentOID;
    private SnmpController sc;
    private SnmpMibManager smm;

    protected javax.swing.Timer timer;
    private int interval = 15;

    public SnmpTreePanel() {
	smm = new SnmpMibManager();

	setLayout(new BorderLayout());

	JTabbedPane jtpl = new JTabbedPane();
	model = new DefaultListModel();
	slist = new JList(model);
	slist.addListSelectionListener(this);
	stree = new JTree();
	jtpl.add("List", new JScrollPane(slist));
	// FIXME implement the tree view
	// jtpl.add("Tree", new JScrollPane(stree));

	JTabbedPane jtpr = new JTabbedPane();

	JPanel jp1 = new JPanel(new BorderLayout());
	tp = new JingleTextPane();
	jp1.add(new JScrollPane(tp), BorderLayout.NORTH);
	// jp2 holds the chart
	jp2 = new JPanel();
	jp1.add(jp2);
	jtpr.add("Details", jp1);
	tpdesc = new JingleTextPane("text/plain");
	jtpr.add("About", new JScrollPane(tpdesc));

	// right hand panel to hold text panel and information
	rpanel = new JPanel(new BorderLayout());

	// split pane to hold the lot
	JSplitPane psplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					   jtpl, jtpr);
	psplit.setOneTouchExpandable(true);
	psplit.setDividerLocation(150);
	add(psplit);
    }

    public void exploreServer(SnmpParams params) {
	this.params = params;
	sc = new SnmpController(params);
	buildData();
	startLoop();
    }

    // FIXME what does this drive?
    public void startLoop() {
	if (timer == null) {
	    timer = new javax.swing.Timer(interval*1000, this);
	}
	timer.start();
    }

    private void setPanel(DefaultMutableTreeNode node) {
	// if ((node != null) && (node.getUserObject() instanceof Kstat)) {
	if (node != null) {
	    // showKstat((Kstat) node.getUserObject());
	}
    }

    private void showObject(SnmpObject sno) {
	// stop any current chart
	if (chart != null) {
	    chart.stopLoop();
	    jp2.removeAll();
	}
	currentOID = sno;
	setText(sno);
	// only if it's a number
	chart = new SnmpChart(sc, sno.toString());
	jp2.add(new ChartPanel(chart.getChart()));
	jp2.validate();
	showMIB(sno.toString());
    }

    private void showMIB(String oid) {
	String s = smm.getNearestNameForOID(oid);
	if (s == null) {
	    tpdesc.setText("");
	} else {
	    tpdesc.setText(s);
	}
    }

    private void setText(SnmpObject sno) {
	StringBuilder sb = new StringBuilder();
	SNMPObject value = sno.getValue();
	sb.append("OID: ").append(sno).append("\n");
	String typeString = value.getClass().getName();
	sb.append("Type: ").append(typeString).append("\n");
	sb.append("Value: ");
	if (typeString.equals("snmp.SNMPOctetString")) {
	    String snmpString = value.toString();
	    // truncate at first null character
	    int nullLocation = snmpString.indexOf('\0');
	    if (nullLocation >= 0)
		snmpString = snmpString.substring(0, nullLocation);
	    sb.append(snmpString);
	    sb.append("  (hex: " + ((SNMPOctetString)value).toHexString() + ")");
	} else {
	    sb.append(value);
	}
	sb.append("\n");
	tp.setText(sb.toString());
    }

    /**
     * Set the update interval
     */
    public void setDelay(int i) {
	interval = i;
	if (chart != null) {
	    chart.setDelay(i);
	}
    }

    private void updateCurrent() {
	if (currentOID != null) {
	    updateOID(currentOID);
	    setText(currentOID);
	}
    }

    private void updateOID(SnmpObject sno) {
	// FIXME these should be shared
	try {
	    InetAddress hostAddress = InetAddress.getByName(params.getServer());
	    SNMPv1CommunicationInterface comInterface =
		new SNMPv1CommunicationInterface(params.getVersion(), hostAddress,
						params.getCommunity());
	    String itemID = sno.getOid().toString();
	    SNMPVarBindList newVars = comInterface.getMIBEntry(itemID);
	    SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
	    SNMPObjectIdentifier snmpOID =
		    (SNMPObjectIdentifier) pair.getSNMPObjectAt(0);
	    SNMPObject snmpValue = pair.getSNMPObjectAt(1);
	    sno.setValue(snmpValue);
	} catch (UnknownHostException uhe) {
	    // FIXME do summat
	} catch (SocketException se) {
	    // FIXE do summat
	} catch (IOException ioe) {
	    // FIXME do summat
	} catch (SNMPBadValueException sbve) {
	    // FIXME do summat
	} catch (SNMPGetException sbve) {
	    // FIXME do summat
	} catch (IndexOutOfBoundsException ioobe) {
	    // FIXME do summat
	    // is this walking off the end?
	}
    }

    // retrieve the data and build the list and tree
    private void buildData() {
	try {
	    InetAddress hostAddress = InetAddress.getByName(params.getServer());
	    SNMPv1CommunicationInterface comInterface =
		new SNMPv1CommunicationInterface(params.getVersion(), hostAddress,
						params.getCommunity());
	    String itemID = "";
	    // start here, but why...
	    String retrievedID = "1.3.6.1.2.1";
	    while (!retrievedID.equals(itemID)) {
		itemID = retrievedID;
		SNMPVarBindList newVars = comInterface.getNextMIBEntry(itemID);
		SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
		SNMPObjectIdentifier snmpOID =
		    (SNMPObjectIdentifier) pair.getSNMPObjectAt(0);
		SNMPObject snmpValue = pair.getSNMPObjectAt(1);
		retrievedID = snmpOID.toString();
		SnmpObject sno = new SnmpObject(snmpOID, snmpValue);
		model.addElement(sno);
	    }
	} catch (UnknownHostException uhe) {
	    // FIXME do summat
	} catch (SocketException se) {
	    // FIXE do summat
	} catch (IOException ioe) {
	    // FIXME do summat
	} catch (SNMPBadValueException sbve) {
	    // FIXME do summat
	} catch (SNMPGetException sbve) {
	    // FIXME do summat
	} catch (IndexOutOfBoundsException ioobe) {
	    // FIXME do summat
	    // is this walking off the end?
	}
    }

    // handle ListSelectionListener events
    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting() == false) {
            if (slist.getSelectedIndex() != -1) {
		Object o = slist.getSelectedValue();
		if (o instanceof SnmpObject) {
		    showObject((SnmpObject) o);
		}
            }
        }
    }

    // handle TreeSelectionListener events
    public void valueChanged(TreeSelectionEvent e) {
	TreePath tpth = e.getNewLeadSelectionPath();
	if (tpth != null) {
	    setPanel((DefaultMutableTreeNode) tpth.getLastPathComponent());
	}
    }

    // handle timer events
    public void actionPerformed(ActionEvent e) {
	updateCurrent();
    }
}
